"""
`sob` is an object serialization/deserialization library intended to facilitate
automated authoring of models for JSON web APIs which are readable and
introspective, and to expedite data validation and integration testing.
"""

from __future__ import annotations

from sob import (
    abc,
    errors,
    hooks,
    meta,
    model,
    properties,
    thesaurus,
    types,
    utilities,
    version,
)
from sob.errors import (
    DeserializeError,
    UnmarshalError,
    UnmarshalTypeError,
    UnmarshalValueError,
    ValidationError,
    VersionError,
)
from sob.hooks import (
    ArrayHooks,
    DictionaryHooks,
    Hooks,
    ObjectHooks,
    get_model_hooks_type,
    get_writable_array_hooks,
    get_writable_dictionary_hooks,
    get_writable_model_hooks,
    get_writable_object_hooks,
    read_array_hooks,
    read_dictionary_hooks,
    read_model_hooks,
    read_object_hooks,
    write_model_hooks,
)
from sob.meta import (
    ArrayMeta,
    DictionaryMeta,
    Meta,
    ObjectMeta,
    Properties,
    get_model_pointer,
    get_model_url,
    get_writable_array_meta,
    get_writable_dictionary_meta,
    get_writable_model_meta,
    get_writable_object_meta,
    read_array_meta,
    read_dictionary_meta,
    read_model_meta,
    read_object_meta,
    set_model_pointer,
    set_model_url,
    version_model,
    write_model_meta,
)
from sob.model import (
    Array,
    Dictionary,
    Model,
    Object,
    deserialize,
    get_model_from_meta,
    get_models_source,
    marshal,
    replace_model_nulls,
    serialize,
    unmarshal,
    validate,
)
from sob.properties import (
    ArrayProperty,
    BooleanProperty,
    BytesProperty,
    DateProperty,
    DateTimeProperty,
    DictionaryProperty,
    EnumeratedProperty,
    IntegerProperty,
    NumberProperty,
    Property,
    StringProperty,
)
from sob.thesaurus import (
    Synonyms,
    Thesaurus,
)
from sob.types import (
    NULL,
    UNDEFINED,
    MutableTypes,
    NoneType,
    Null,
    Types,
    Undefined,
)
from sob.version import Version

__all__: tuple[str, ...] = (
    # Modules
    "utilities",
    "abc",
    "model",
    "errors",
    "properties",
    "meta",
    "types",
    "hooks",
    "request",
    "thesaurus",
    "version",
    # Functions and Classes
    "Model",
    "Object",
    "Dictionary",
    "Array",
    "get_model_from_meta",
    "marshal",
    "unmarshal",
    "serialize",
    "deserialize",
    "validate",
    "replace_model_nulls",
    "get_models_source",
    "Meta",
    "ObjectMeta",
    "Properties",
    "read_model_meta",
    "read_object_meta",
    "read_array_meta",
    "read_dictionary_meta",
    "get_writable_model_meta",
    "get_writable_object_meta",
    "get_writable_array_meta",
    "get_writable_dictionary_meta",
    "write_model_meta",
    "get_model_pointer",
    "set_model_pointer",
    "set_model_url",
    "get_model_url",
    "version_model",
    "DictionaryMeta",
    "ArrayMeta",
    "Hooks",
    "ObjectHooks",
    "DictionaryHooks",
    "ArrayHooks",
    "get_model_hooks_type",
    "get_writable_model_hooks",
    "get_writable_object_hooks",
    "get_writable_array_hooks",
    "get_writable_dictionary_hooks",
    "read_model_hooks",
    "read_object_hooks",
    "read_array_hooks",
    "read_dictionary_hooks",
    "write_model_hooks",
    "Property",
    "StringProperty",
    "DateProperty",
    "DateTimeProperty",
    "BytesProperty",
    "EnumeratedProperty",
    "NumberProperty",
    "IntegerProperty",
    "BooleanProperty",
    "ArrayProperty",
    "DictionaryProperty",
    "Synonyms",
    "Thesaurus",
    "Null",
    "NULL",
    "Undefined",
    "UNDEFINED",
    "Types",
    "MutableTypes",
    "NoneType",
    "Version",
    "VersionError",
    "DeserializeError",
    "UnmarshalError",
    "UnmarshalTypeError",
    "UnmarshalValueError",
    "ValidationError",
)
